@@ -25,10 +25,12 @@ from dstack.backend.base.logs import fix_urls
 from dstack.cli.commands import BasicCommand
 from dstack.cli.commands.run.ssh_tunnel import allocate_local_ports, run_ssh_tunnel
 from dstack.cli.common import console, print_runs
+from dstack.cli.config import BaseConfig
 from dstack.core.error import check_backend, check_config, check_git
 from dstack.core.job import Job, JobHead, JobStatus
 from dstack.core.repo import RepoAddress
 from dstack.core.request import RequestStatus
+from dstack.core.userconfig import RepoUserConfig
 from dstack.utils.common import since
 
 __all__ = "RunCommand"
@@ -141,7 +143,11 @@ def poll_logs_ws(backend: Backend, repo_address: RepoAddress, job: Job, ports: D
 
 
 def poll_run(
-    repo_address: RepoAddress, job_heads: List[JobHead], backend: Backend, ssh_key: Optional[str]
+    repo_address: RepoAddress,
+    job_heads: List[JobHead],
+    backend: Backend,
+    ssh_key: Optional[str],
+    openssh_server: bool,
 ):
     run_name = job_heads[0].run_name
     try:
@@ -188,7 +194,7 @@ def poll_run(
         jobs = [backend.get_job(repo_address, job_head.job_id) for job_head in job_heads]
         ports = {}
         if backend.name != "local":
-            console.print("Starting SSH tunnel... It may take a while.")
+            console.print("Starting SSH tunnel...")
             ports = allocate_local_ports(jobs)
             if not run_ssh_tunnel(
                 ssh_key, jobs[0].host_name, ports
@@ -200,6 +206,14 @@ def poll_run(
         console.print("[grey58]To interrupt, press Ctrl+C.[/]")
         console.print()
 
+        if openssh_server:
+            ssh_port = jobs[0].ports[-1]
+            ssh_port = ports.get(ssh_port, ssh_port)
+            ssh_key_escaped = ssh_key.replace(" ", "\\ ")
+            console.print("To connect via SSH, use:")
+            console.print(f"  ssh -i {ssh_key_escaped} root@localhost -p {ssh_port}")
+            console.print()
+
         run = backend.list_run_heads(repo_address, run_name)[0]
         if len(job_heads) == 1 and run and run.status == JobStatus.RUNNING:
             poll_logs_ws(backend, repo_address, jobs[0], ports)
@@ -273,6 +287,7 @@ class RunCommand(BasicCommand):
             self._parser.print_help()
             exit(1)
         try:
+            config = BaseConfig()
             repo_data = load_repo_data()
             backend = get_local_backend()
             if args.remote is not None:
@@ -300,14 +315,24 @@ class RunCommand(BasicCommand):
                 sys.exit()
 
             repo_credentials = backend.get_repo_credentials(repo_data)
+            repo_user_config = config.read(
+                config.repos / f"{repo_data.path(delimiter='.')}.yaml",
+                RepoUserConfig,
+                non_empty=False,
+            )
             if not repo_credentials:
                 sys.exit(f"Call `dstack init` first")
-            if backend != "local" and not args.detach:
-                if not repo_credentials.ssh_key_path:
+            if not repo_user_config.ssh_key_path:
+                if (
+                    (backend.name != "local" and not args.detach)
+                    or workflow_data.get("ssh", False)
+                    or "--ssh" in provider_args
+                ):
                     console.print("Call `dstack init` first")
+                    console.print("  [gray58]No valid SSH identity[/]")
                     exit(1)
-                else:
-                    workflow_data["ssh_key_pub"] = _read_ssh_key_pub(repo_credentials.ssh_key_path)
+            else:
+                workflow_data["ssh_key_pub"] = _read_ssh_key_pub(repo_user_config.ssh_key_path)
 
             run_name = backend.create_run(repo_data)
             provider.load(backend, provider_args, workflow_name, workflow_data, run_name)
@@ -319,7 +344,13 @@ class RunCommand(BasicCommand):
             backend.update_repo_last_run_at(repo_data, last_run_at=int(round(time.time() * 1000)))
             print_runs(list_runs_with_merged_backends([backend], run_name=run_name))
             if not args.detach:
-                poll_run(repo_data, jobs, backend, ssh_key=repo_credentials.ssh_key_path)
+                poll_run(
+                    repo_data,
+                    jobs,
+                    backend,
+                    ssh_key=repo_user_config.ssh_key_path,
+                    openssh_server=provider.openssh_server,
+                )
         except ValidationError as e:
             sys.exit(
                 f"There a syntax error in one of the files inside the {os.getcwd()}/.dstack/workflows directory:\n\n{e}"
