@@ -44,6 +44,7 @@ def ask_choice(
             search_key=None,
             search_highlight_style=["fg_purple"],
             cursor_index=cursor_index,
+            raise_error_on_interrupt=True,
         )
         chosen_menu_index = terminal_menu.show()
         chosen_menu_label = labels[chosen_menu_index].replace("[", "\\[")
@@ -70,7 +71,7 @@ def ask_choice(
                 return ask_choice(title, labels, values, selected_value, show_choices)
 
 
-def print_runs(runs_with_backends: List[Tuple[RunHead, List[Backend]]]):
+def generate_runs_table(runs_with_backends: List[Tuple[RunHead, List[Backend]]]):
     table = Table(box=None)
     table.add_column("RUN", style="bold", no_wrap=True)
     table.add_column("WORKFLOW", style="grey58", max_width=16)
@@ -91,7 +92,11 @@ def print_runs(runs_with_backends: List[Tuple[RunHead, List[Backend]]]):
             _status_color(run, run.tag_name or "", False, False),
             _status_color(run, ", ".join(b.name for b in backends), False, False),
         )
+    return table
 
+
+def print_runs(runs_with_backends: List[Tuple[RunHead, List[Backend]]]):
+    table = generate_runs_table(runs_with_backends)
     console.print(table)
 
 
